#ifdef HAVE_CONFIG_H
#include <config.h>
#endif /* HAVE_CONFIG_H */
#include        "sadie.h"

/*-Copyright Information------------------------------------------------------*/
/* Copyright (c) 1988 by the University of Arizona Digital Image Analysis Lab */
/*----------------------------------------------------------------------------*/
/*-General Information--------------------------------------------------------*/
/*                                                                            */
/*   This function separates one multi-band image into individual             */
/*   single-band images (for disk or tape I/O).                               */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/*-Interface Information------------------------------------------------------*/
void BILTOBSQ (
IMAGE *in,      /*  I   Pointer to the input image.                           */
IMAGE **out     /*  O   Pointer to an array of pointers to the output images. */
/*----------------------------------------------------------------------------*/
) { register uint32_t i, j, k;
    char   msg[SLEN];
    double pinc, psum;

    if (TIMES) TIMING(T_BILTOBSQ);
    if (NAMES) {
        MESSAGE('I',"");
        MESSAGE('I',"BILTOBSQ");
    }

    /* check input */
    if (!CHECKIMG(in)) {
        MESSAGE('E'," Can't identify image.");
        goto the_end;
    } else if (in->nbnd < 2) {
        MESSAGE('E'," Input image must be multi-band.");
        goto the_end;
    }

    if (NAMES) {
        MESSAGE('I',"");
        sprintf(msg," Input image:  %s",in->text);
        MESSAGE('I',msg);
        MESSAGE('I'," ...............");
    }

    /* create images of appropriate size */
    for (i=0; i<in->nbnd; i++) {
        if (!CHECKIMG(out[i])) GETMEM(1,in->nlin,in->npix,&out[i]);
        if (!out[i]) goto the_end;
    }

    /* initialize progress indicator */
    if (LINES  &&  !PROGRESS(psum=0.0)) goto the_end;
    pinc = 1.0/(double)in->nbnd/(double)in->nlin;

    /* separate images */
    for (i=0; i<in->nbnd; i++) {
        for (j=0; j<in->nlin; j++) {
            for (k=0; k<in->npix; k++) {
                out[i]->data[0][j][k] = in->data[i][j][k];
            }
            if (LINES  &&  !PROGRESS(psum+=pinc)) goto the_end;
        }
    }

    the_end:
    if (TIMES) TIMING(T_EXIT);
}
