#ifdef HAVE_CONFIG_H
#include <config.h>
#endif /* HAVE_CONFIG_H */
#include        "sadie.h"

/*-Copyright Information------------------------------------------------------*/
/* Copyright (c) 2004 by the University of Arizona Digital Image Analysis Lab */
/*----------------------------------------------------------------------------*/
/*-General Information--------------------------------------------------------*/
/*                                                                            */
/*   This function extracts the specified bit plane from a single-band image. */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/*-Interface Information------------------------------------------------------*/
void BITPLANE (
IMAGE *in,         /*  I   Pointer to the input image.                        */
uint32_t bitplane, /*  I   Index [0 .. 31] of the bit plane to extract.       */
IMAGE **out        /*  O   Address of a pointer to the output image.          */
/*----------------------------------------------------------------------------*/
) { register uint32_t j, k, bitmask;
    char msg[SLEN];

    if (TIMES) TIMING(T_BITPLANE);
    if (NAMES) {
        MESSAGE('I',"");
        MESSAGE('I',"BITPLANE");
    }

    /* check input */
    if (!CHECKIMG(in)) {
        MESSAGE('E'," Can't identify image.");
        goto the_end;
    } else if (in->nbnd > 1) {
        MESSAGE('E'," Input image must be single-band.");
        goto the_end;
    } else if (bitplane > 31) {
        MESSAGE('E'," Bitplane value must be between 0 and 31.");
        goto the_end;
    }

    if (NAMES) {
        MESSAGE('I',"");
        sprintf(msg," Input image:  %s",in->text);
        MESSAGE('I',msg);
#ifndef SADIE_IPT
        sprintf(msg," Bit plane:    %lu",bitplane);
#else
        sprintf(msg," Bit plane:    %lu",bitplane+1);
#endif
        MESSAGE('I',msg);
        MESSAGE('I'," ...............");
    }

    /* create image of appropriate size */
    if (!CHECKIMG(*out)) GETMEM(1,in->nlin,in->npix,out);
    if (!*out) goto the_end;

    /* compute bit mask */
    bitmask = 0x00000001 << bitplane;

    /* extract bit plane */
    for (j=0; j<in->nlin; j++) {
        for (k=0; k<in->npix; k++) {
            (*out)->data[0][j][k] = (PIXEL)(((int32_t)in->data[0][j][k]) & bitmask);
        }
    }

    the_end:
    if (TIMES) TIMING(T_EXIT);
}
