#ifdef HAVE_CONFIG_H
#include <config.h>
#endif /* HAVE_CONFIG_H */
#include        "sadie.h"

/*-Copyright Information------------------------------------------------------*/
/* Copyright (c) 1993 by the University of Arizona Digital Image Analysis Lab */
/*----------------------------------------------------------------------------*/
/*-General Information--------------------------------------------------------*/
/*                                                                            */
/*   This function creates a chirp test pattern.                              */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/*-Interface Information------------------------------------------------------*/
void CHIRP (
uint32_t nlin,  /*  I   Number of lines in the output image.                  */
uint32_t npix,  /*  I   Number of pixels/line in the output image.            */
double period,  /*  I   Period of the chirp in pixels.                        */
IMAGE  **out    /*  O   Address of a pointer to the output image.             */
/*----------------------------------------------------------------------------*/
) { register uint32_t j, k;
    char   msg[SLEN];
    double x, y;

    if (TIMES) TIMING(T_CHIRP);
    if (NAMES) {
        MESSAGE('I',"");
        MESSAGE('I',"CHIRP");
        MESSAGE('I',"");
        sprintf(msg," Image size: lines:           %lu",nlin);
        MESSAGE('I',msg);
        sprintf(msg,"             pixels:          %lu",npix);
        MESSAGE('I',msg);
        sprintf(msg," Period:                    %12.4e",period);
        MESSAGE('I',msg);
        MESSAGE('I'," ...............");
    }

    /* create image of appropriate size */
    if (!CHECKIMG(*out)) GETMEM(1,nlin,npix,out);

    /* create chirp test pattern */
    period = PI / period;
    for (j=0; j<nlin; j++) {
    	y = ((double)j-(double)(nlin/2))*((double)j-(double)(nlin/2));
        for (k=0; k<npix; k++) {
            x = sqrt(((double)k-(double)(npix/2))*((double)k-(double)(npix/2)) + y);
            (*out)->data[0][j][k] = (PIXEL)cos(x*x*period);
        }
    }

    the_end:
    if (TIMES) TIMING(T_EXIT);
}
