#ifdef HAVE_CONFIG_H
#include <config.h>
#endif /* HAVE_CONFIG_H */
#include        "sadie.h"

/*-Copyright Information------------------------------------------------------*/
/* Copyright (c) 1990 by the University of Arizona Digital Image Analysis Lab */
/*----------------------------------------------------------------------------*/
/*-General Information--------------------------------------------------------*/
/*                                                                            */
/*   This function compares two images, and creates a binary mask as the      */
/*   output image.                                                            */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/*-Interface Information------------------------------------------------------*/
void COMPARE (
IMAGE *in1,     /*  I   Pointer to the first input image.                     */
IMAGE *in2,     /*  I   Pointer to the second input image.                    */
short option,   /*  I   Switch, indicating type of comparison operation:      */
                /*      LT    -   Less than.                                  */
                /*      LE    -   Less than or equal to.                      */
                /*      GT    -   Greater than.                               */
                /*      GE    -   Greater than or equal to.                   */
                /*      EQ    -   Equal to.                                   */
                /*      NE    -   Not equal to.                               */
                /*      AND   -   Logical and.                                */
                /*      OR    -   Logical or.                                 */
                /*      XOR   -   Logical xor.                                */
                /*      NOT   -   Logical not (of the first input image).     */
IMAGE **out     /*  O   Address of a pointer to the output image.             */
/*----------------------------------------------------------------------------*/
) { register uint32_t i, j, k;
    char msg[SLEN];

    if (TIMES) TIMING(T_COMPARE);
    if (NAMES) {
        MESSAGE('I',"");
        MESSAGE('I',"COMPARE");
    }

    /* check input */
    if (!CHECKIMG(in1)) {
        MESSAGE('E'," Can't identify first input image.");
        goto the_end;
    } else if (option != NOT  &&  !CHECKIMG(in2)) {
        MESSAGE('E'," Can't identify second input image.");
        goto the_end;
    } else if (option != NOT  &&  in1->nbnd != in2->nbnd) {
        MESSAGE('E'," Number of bands must be identical in both images.");
        goto the_end;
    } else if (option != NOT  &&  in1->nlin != in2->nlin) {
        MESSAGE('E'," Number of lines must be identical in both images.");
        goto the_end;
    } else if (option != NOT  &&  in1->npix != in2->npix) {
        MESSAGE('E'," Number of pixels/line must be identical in both images.");
        goto the_end;
    } else if (option != LT  &&  option != LE  &&  option != GT  &&  option != GE  &&  option != EQ  &&  option != NE  &&  option != AND  &&  option != OR  &&  option != XOR  &&  option != NOT) {
        MESSAGE('E'," Option must be one of LT, LE, GT, GE, EQ, NE, AND, OR, XOR, or NOT.");
        goto the_end;
    }

    if (NAMES) {
        MESSAGE('I',"");
        sprintf(msg," First input image:    %s",in1->text);
        MESSAGE('I',msg);
        sprintf(msg," Second input image:   %s",in2->text);
        MESSAGE('I',msg);
        if (option == LT) {
            MESSAGE('I'," Operation:            Less than");
        } else if (option == LE) {
            MESSAGE('I'," Operation:            Less than or equal to");
        } else if (option == GT) {
            MESSAGE('I'," Operation:            Greater than");
        } else if (option == GE) {
            MESSAGE('I'," Operation:            Greater than or equal to");
        } else if (option == EQ) {
            MESSAGE('I'," Operation:            Equal to");
        } else if (option == NE) {
            MESSAGE('I'," Operation:            Not equal to");
        } else if (option == AND) {
            MESSAGE('I'," Operation:            Logical and");
        } else if (option == OR) {
            MESSAGE('I'," Operation:            Logical or");
        } else if (option == XOR) {
            MESSAGE('I'," Operation:            Logical xor");
        } else if (option == NOT) {
            MESSAGE('I'," Operation:            Logical not (of the first input image)");
        }
        MESSAGE('I'," ...............");
    }

    /* create image of appropriate size */
    if (!CHECKIMG(*out)) GETMEM(in1->nbnd,in1->nlin,in1->npix,out);
    if (!*out) goto the_end;

    /* create mask */
    switch (option) {
    case LT:
        for (i=0; i<in1->nbnd; i++) {
            for (j=0; j<in1->nlin; j++) {
                for (k=0; k<in1->npix; k++) {
                    if (in1->data[i][j][k] < in2->data[i][j][k]) {
                        (*out)->data[i][j][k] = (PIXEL)1;
                    } else {
                        (*out)->data[i][j][k] = (PIXEL)0;
                    }
                }
            }
        }
        break; 
     case LE:
        for (i=0; i<in1->nbnd; i++) {
            for (j=0; j<in1->nlin; j++) {
                for (k=0; k<in1->npix; k++) {
                    if (in1->data[i][j][k] <= in2->data[i][j][k]) {
                        (*out)->data[i][j][k] = (PIXEL)1;
                    } else {
                        (*out)->data[i][j][k] = (PIXEL)0;
                    }
                }
            }
        }
        break; 
    case GT:
        for (i=0; i<in1->nbnd; i++) {
            for (j=0; j<in1->nlin; j++) {
                for (k=0; k<in1->npix; k++) {
                    if (in1->data[i][j][k] > in2->data[i][j][k]) {
                        (*out)->data[i][j][k] = (PIXEL)1;
                    } else {
                        (*out)->data[i][j][k] = (PIXEL)0;
                    }
                }
            }
        }
        break; 
    case GE:
        for (i=0; i<in1->nbnd; i++) {
            for (j=0; j<in1->nlin; j++) {
                for (k=0; k<in1->npix; k++) {
                    if (in1->data[i][j][k] >= in2->data[i][j][k]) {
                        (*out)->data[i][j][k] = (PIXEL)1;
                    } else {
                        (*out)->data[i][j][k] = (PIXEL)0;
                    }
                }
            }
        }
        break; 
    case EQ:
        for (i=0; i<in1->nbnd; i++) {
            for (j=0; j<in1->nlin; j++) {
                for (k=0; k<in1->npix; k++) {
                    if (in1->data[i][j][k] == in2->data[i][j][k]) {
                        (*out)->data[i][j][k] = (PIXEL)1;
                    } else {
                        (*out)->data[i][j][k] = (PIXEL)0;
                    }
                }
            }
        }
        break; 
    case NE:
        for (i=0; i<in1->nbnd; i++) {
            for (j=0; j<in1->nlin; j++) {
                for (k=0; k<in1->npix; k++) {
                    if (in1->data[i][j][k] != in2->data[i][j][k]) {
                        (*out)->data[i][j][k] = (PIXEL)1;
                    } else {
                        (*out)->data[i][j][k] = (PIXEL)0;
                    }
                }
            }
        }
        break; 
    case AND:
        for (i=0; i<in1->nbnd; i++) {
            for (j=0; j<in1->nlin; j++) {
                for (k=0; k<in1->npix; k++) {
                    if (in1->data[i][j][k] != (PIXEL)0  &&  in2->data[i][j][k] != (PIXEL)0) {
                        (*out)->data[i][j][k] = (PIXEL)1;
                    } else {
                        (*out)->data[i][j][k] = (PIXEL)0;
                    }
                }
            }
        }
        break; 
    case OR:
        for (i=0; i<in1->nbnd; i++) {
            for (j=0; j<in1->nlin; j++) {
                for (k=0; k<in1->npix; k++) {
                    if (in1->data[i][j][k] != (PIXEL)0  ||  in2->data[i][j][k] != (PIXEL)0) {
                        (*out)->data[i][j][k] = (PIXEL)1;
                    } else {
                        (*out)->data[i][j][k] = (PIXEL)0;
                    }
                }
            }
        }
        break; 
    case XOR:
        for (i=0; i<in1->nbnd; i++) {
            for (j=0; j<in1->nlin; j++) {
                for (k=0; k<in1->npix; k++) {
                if (in1->data[i][j][k] != (PIXEL)0  &&  in2->data[i][j][k] == (PIXEL)0   ||   in1->data[i][j][k] == (PIXEL)0  &&  in2->data[i][j][k] != (PIXEL)0) {
                        (*out)->data[i][j][k] = (PIXEL)1;
                    } else {
                        (*out)->data[i][j][k] = (PIXEL)0;
                    }
                }
            }
        }
        break; 
    case NOT:
        for (i=0; i<in1->nbnd; i++) {
            for (j=0; j<in1->nlin; j++) {
                for (k=0; k<in1->npix; k++) {
                    if (in1->data[i][j][k] == (PIXEL)0) {
                        (*out)->data[i][j][k] = (PIXEL)1;
                    } else {
                        (*out)->data[i][j][k] = (PIXEL)0;
                    }
                }
            }
        }
        break; 
    }

    the_end:
    if (TIMES) TIMING(T_EXIT);
}
