#ifdef HAVE_CONFIG_H
#include <config.h>
#endif /* HAVE_CONFIG_H */
#include        "sadie.h"

/*-Copyright Information------------------------------------------------------*/
/* Copyright (c) 1988 by the University of Arizona Digital Image Analysis Lab */
/*----------------------------------------------------------------------------*/
/*-General Information--------------------------------------------------------*/
/*                                                                            */
/*   This function produces a contour map of an image.                        */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/*-Interface Information------------------------------------------------------*/
void CONTOUR (
IMAGE  *in,     /*  I   Pointer to the input image.                           */
short  nlev,    /*  I   Number of contour lines.                              */
double width,   /*  I   Width of contour lines (fraction, less than one).     */
IMAGE  **out    /*  O   Address of a pointer to the output image.             */
                /*      The input image may be overwritten (out = &in).       */
/*----------------------------------------------------------------------------*/
) { register uint32_t i, j, k;
    char   msg[SLEN];
    double delta, class;

    if (TIMES) TIMING(T_CONTOUR);
    if (NAMES) {
        MESSAGE('I',"");
        MESSAGE('I',"CONTOUR");
    }

    /* check input */
    if (!CHECKIMG(in)) {
        MESSAGE('E'," Can't identify image.");
        goto the_end;
    } else if (nlev <= 0) {
        MESSAGE('E'," Number of contours must be greater than zero.");
        goto the_end;
    } else if (0.0 >= width  ||  width >= 1.0) {
        MESSAGE('E'," Width of contour lines must be greater than zero and less than one.");
        goto the_end;
    }

    if (NAMES) {
        MESSAGE('I',"");
        sprintf(msg," Input image:               %s",in->text);
        MESSAGE('I',msg);
        sprintf(msg," Number of contour lines:   %d",nlev);
        MESSAGE('I',msg);
        sprintf(msg," Width of contour lines: %7.2f",width);
        MESSAGE('I',msg);
        MESSAGE('I'," ...............");
    }

    /* create image of appropriate size */
    if (!CHECKIMG(*out)) GETMEM(in->nbnd,in->nlin,in->npix,out);
    if (!*out) goto the_end;

    /* compute contours */
    RANGE(in);
    width = (1.0-width)/2.0;
    delta = (double)(in->gmax-in->gmin)/(double)nlev;
    for (i=0; i<in->nbnd; i++) {
        for (j=0; j<in->nlin; j++) {
            for (k=0; k<in->npix; k++) {
                class = (double)(in->data[i][j][k]-in->gmin)/delta;
                (*out)->data[i][j][k] = (PIXEL)(fabs(class-(double)rnd(class)) > width  ?  SMAX : SMIN);
            }
        }
    }
    (*out)->gmin = (PIXEL)SMIN;
    (*out)->gmax = (PIXEL)SMAX;

    the_end:
    if (TIMES) TIMING(T_EXIT);
}
