#ifdef HAVE_CONFIG_H
#include <config.h>
#endif /* HAVE_CONFIG_H */
#include        "sadie.h"

/*-Copyright Information------------------------------------------------------*/
/* Copyright (c) 1988 by the University of Arizona Digital Image Analysis Lab */
/*----------------------------------------------------------------------------*/
/*-General Information--------------------------------------------------------*/
/*                                                                            */
/*   This low-level function computes a histograms for all image bands using  */
/*   only data in the specified range.                                        */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/*-Interface Information------------------------------------------------------*/
void HISTOGRM (
IMAGE *in,      /*  I   Pointer to the image.                                 */
uint32_t inc,   /*  I   Pixels/line and line increment.                       */
short option,   /*  I   Switch, indicating how to use input pixel range:      */
                /*      INCLUDE - Use only this graylevel range for computing */
                /*                histogram.                                  */
                /*      EXCLUDE - Exclude this graylevel range.               */
                /*      IGNORE  - Do not use graylevel range.                 */
PIXEL gmin,     /*  I   Minimum graylevel in range.                           */
PIXEL gmax,     /*  I   Maximum graylevel in range.                           */
uint32_t nlev,  /*  I   Number of bins in the histogram.                      */
uint64_t *hist  /*  O   Pointer to the histogram array[in->nbnd*nlev].        */
/*----------------------------------------------------------------------------*/
) { register uint32_t i, j, k;
    uint64_t index;
    double factor;

    if (TIMES) TIMING(T_HISTOGRM);

    if (option == IGNORE  ||  option == INCLUDE) {
        factor = (double)nlev/(double)(gmax-gmin+1);
    } else {
        RANGE(in);
        factor = (double)nlev/(double)(in->gmax-in->gmin+1);
    }

    /* compute histogram for each band, clip if necessary */
    for (i=0; i<in->nbnd*nlev; i++) {
        hist[i] = 0;
    }
    if (option == IGNORE) {
        for (i=0; i<in->nbnd; i++) {
            for (j=0; j<in->nlin; j+=inc) {
                for (k=0; k<in->npix; k+=inc) {
                    if (in->data[i][j][k] <= gmin) {
                        index = (uint64_t)i*(uint64_t)nlev;
                    } else if (gmin < in->data[i][j][k]  &&  in->data[i][j][k] < gmax) {
                        index = (uint64_t)i*(uint64_t)nlev + (uint64_t)((double)(in->data[i][j][k]-gmin)*factor);
                    } else {
                        index = (uint64_t)i*(uint64_t)nlev + (uint64_t)nlev - 1;
                    }
                    hist[index] += 1;
                }
            }
        }
    } else if (option == INCLUDE) {
        for (i=0; i<in->nbnd; i++) {
            for (j=0; j<in->nlin; j+=inc) {
                for (k=0; k<in->npix; k+=inc) {
                    if (in->data[i][j][k] == gmin) {
                        index = (uint64_t)i*(uint64_t)nlev;
                    } else if (gmin < in->data[i][j][k]  &&  in->data[i][j][k] < gmax) {
                        index = (uint64_t)i*(uint64_t)nlev + (uint64_t)((double)(in->data[i][j][k]-gmin)*factor);
                    } else if (in->data[i][j][k] == gmax) {
                        index = (uint64_t)i*(uint64_t)nlev + (uint64_t)nlev - 1;
                    }
                    hist[index] += 1;
                }
            }
        }
    } else {
        for (i=0; i<in->nbnd; i++) {
            for (j=0; j<in->nlin; j+=inc) {
                for (k=0; k<in->npix; k+=inc) {
                    if (in->data[i][j][k] < gmin || in->data[i][j][k] > gmax) {
                        if (in->data[i][j][k] == in->gmin) {
                            index = (uint64_t)i*(uint64_t)nlev;
                        } else if (in->data[i][j][k] == in->gmax) {
                            index = (uint64_t)i*(uint64_t)nlev + (uint64_t)nlev - 1;
                        } else {
                            index = (uint64_t)i*(uint64_t)nlev + (uint64_t)((double)(in->data[i][j][k]-in->gmin)*factor);
                        }
                        hist[index] += 1;
                    }
                }
            }
        }
    }

    the_end:
    if (TIMES) TIMING(T_EXIT);
}
