#ifdef HAVE_CONFIG_H
#include <config.h>
#endif /* HAVE_CONFIG_H */
#include "sadie.h"

#if defined SADIE_MACOS
#include <TextUtils.h>
#include <Files.h>
#endif

/*-Copyright Information------------------------------------------------------*/
/* Copyright (c) 1994 by the University of Arizona Digital Image Analysis Lab */
/*----------------------------------------------------------------------------*/
/*-General Information--------------------------------------------------------*/
/*                                                                            */
/*   This function writes an EPSF image to disk.                              */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/*-Interface Information------------------------------------------------------*/
void IMG2EPSF (
IMAGE *in,           /*  I   Pointer to the image.                            */
unsigned char *name, /*  I   String, containing the name of the disk file.    */
uint32_t nbit,       /*  I   Bits/pixel in disk file.                         */
short ref,           /*  I   Reference number for the disk file.              */
short vref           /*  I   Volume reference number for the disk file.       */
/*----------------------------------------------------------------------------*/
) { static char hex[] = { '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F' };
    register uint32_t i, j, k;
    uint64_t total;
    unsigned char *ptr, *data, cval;
    char msg[SLEN];
    short error;
    long nbytes;
    double factor, dval;
    time_t secs;
    struct tm *t;
#if defined SADIE_UNIX
    FILE *fp = NULL;
#elif defined SADIE_MACOS
    Str255 pstr;
#endif

    if (TIMES) TIMING(T_IMG2EPSF);
    if (NAMES) {
        MESSAGE('I',"");
        MESSAGE('I',"IMG2EPSF");
    }

    /* check input */
    if (!CHECKIMG(in)) {
        MESSAGE('E'," Can't identify image.");
        goto the_end;
    }

    if (NAMES) {
        MESSAGE('I',"");
        sprintf(msg," Input image:               %s",in->text);
        MESSAGE('I',msg);
        sprintf(msg," Output disk file:          %s",name);
        MESSAGE('I',msg);
        sprintf(msg," Output format:             %lu-bit EPSF",nbit);
        MESSAGE('I',msg);
        MESSAGE('I'," ...............");
    }

#if defined SADIE_UNIX
    /* open image file */
    if (!(fp=fopen(name,"w"))) {
        sprintf(msg," Can't open file %s.",name);
        MESSAGE('E',msg);
        goto the_end;
    }
#endif

    total = max((uint64_t)4096,(uint64_t)2*(uint64_t)in->npix+(uint64_t)1) * (uint64_t)sizeof(unsigned char);
    if (total > (uint64_t)SIZE_MAX  ||  (ptr=data=(unsigned char *)malloc((size_t)total)) == NULL) {
        MESSAGE('E'," Memory request failed.");
        goto the_end;
    }
    secs = time(NULL);
    t = localtime(&secs);

    /* write image header to file */
    sprintf((char *)ptr                    ,"%%!PS-Adobe-3.0 EPSF-3.0\r");
    sprintf((char *)ptr+strlen((char *)ptr),"%%%%Creator: %s\r",LOCALE);
    sprintf((char *)ptr+strlen((char *)ptr),"%%%%Title: %s\r",name);
    sprintf((char *)ptr+strlen((char *)ptr),"%%%%CreationDate: %02d/%02d/%02d %02d:%02d\r",t->tm_mon+1,t->tm_mday,t->tm_year%100,t->tm_hour,t->tm_min);
    sprintf((char *)ptr+strlen((char *)ptr),"%%%%BoundingBox: 0 0 %lu %lu\r",in->npix,in->nlin);
    sprintf((char *)ptr+strlen((char *)ptr),"%%%%SuppressDotGainCompensation\r");
    if (in->nbnd != 3) {
        sprintf((char *)ptr+strlen((char *)ptr),"%%%%DocumentProcessColors: Black\r");
    }
    sprintf((char *)ptr+strlen((char *)ptr),"%%%%EndComments\r");
    sprintf((char *)ptr+strlen((char *)ptr),"%%%%BeginProlog\r");
    sprintf((char *)ptr+strlen((char *)ptr),"%%%%EndProlog\r");
    sprintf((char *)ptr+strlen((char *)ptr),"%%%%BeginSetup\r");
    sprintf((char *)ptr+strlen((char *)ptr),"%%%%EndSetup\r");
    sprintf((char *)ptr+strlen((char *)ptr),"%%ImageData: %lu %lu %lu %lu %lu %lu %lu \"beginimage\"\r",in->npix,in->nlin,nbit,in->nbnd,(uint32_t)in->nbnd==3,in->npix,(uint32_t)1);
    if (in->nbnd != 3) {
        sprintf((char *)ptr+strlen((char *)ptr),"gsave\r");
        sprintf((char *)ptr+strlen((char *)ptr),"40 dict begin\r");
        sprintf((char *)ptr+strlen((char *)ptr),"/_image systemdict /image get def\r");
        sprintf((char *)ptr+strlen((char *)ptr),"/_setgray systemdict /setgray get def\r");
        sprintf((char *)ptr+strlen((char *)ptr),"/_currentgray systemdict /currentgray get def\r");
        sprintf((char *)ptr+strlen((char *)ptr),"/_settransfer systemdict /settransfer get def\r");
        sprintf((char *)ptr+strlen((char *)ptr),"/_currenttransfer systemdict /currenttransfer get def\r");
        sprintf((char *)ptr+strlen((char *)ptr),"/blank 0 _currenttransfer exec\r");
        sprintf((char *)ptr+strlen((char *)ptr),"1 _currenttransfer exec eq def\r");
        sprintf((char *)ptr+strlen((char *)ptr),"/negative blank\r");
        sprintf((char *)ptr+strlen((char *)ptr),"{0 _currenttransfer exec 0.5 lt}\r");
        sprintf((char *)ptr+strlen((char *)ptr),"{0 _currenttransfer exec 1 _currenttransfer exec gt}\r");
        sprintf((char *)ptr+strlen((char *)ptr),"ifelse def\r");
        sprintf((char *)ptr+strlen((char *)ptr),"/inverted? negative def\r");
        sprintf((char *)ptr+strlen((char *)ptr),"/level2 systemdict /languagelevel known\r");
        sprintf((char *)ptr+strlen((char *)ptr),"{languagelevel 2 ge} {false} ifelse def\r");
        sprintf((char *)ptr+strlen((char *)ptr),"gsave\r");
        sprintf((char *)ptr+strlen((char *)ptr),"/rows %lu def\r",in->nlin);
        sprintf((char *)ptr+strlen((char *)ptr),"/cols %lu def\r",in->npix);
        sprintf((char *)ptr+strlen((char *)ptr),"72 %llu mul %llu div dup cols mul exch rows mul scale\r",(uint64_t)in->nlin*(uint64_t)in->npix,(uint64_t)72*(uint64_t)in->nlin*(uint64_t)in->npix);
        sprintf((char *)ptr+strlen((char *)ptr),"level2 {\r");
        sprintf((char *)ptr+strlen((char *)ptr),"/DeviceGray\r");
        sprintf((char *)ptr+strlen((char *)ptr),"setcolorspace} if\r");
        sprintf((char *)ptr+strlen((char *)ptr),"/picstr1 %lu string def\r",in->npix);
        sprintf((char *)ptr+strlen((char *)ptr),"/readdata {currentfile exch readhexstring pop} def\r");
        sprintf((char *)ptr+strlen((char *)ptr),"/image2 level2 {/image load def} {{begin\r");
        sprintf((char *)ptr+strlen((char *)ptr),"Width Height BitsPerComponent ImageMatrix\r");
        sprintf((char *)ptr+strlen((char *)ptr),"/DataSource load image end} def} ifelse\r");
        sprintf((char *)ptr+strlen((char *)ptr),"/beginimage {\r");
        sprintf((char *)ptr+strlen((char *)ptr),"image2\r");
        sprintf((char *)ptr+strlen((char *)ptr),"} def\r");
        sprintf((char *)ptr+strlen((char *)ptr),"12 dict begin\r");
        sprintf((char *)ptr+strlen((char *)ptr),"/ImageType 1 def\r");
        sprintf((char *)ptr+strlen((char *)ptr),"/Width cols def\r");
        sprintf((char *)ptr+strlen((char *)ptr),"/Height rows def\r");
        sprintf((char *)ptr+strlen((char *)ptr),"/ImageMatrix [cols 0 0 rows neg 0 rows] def\r");
        sprintf((char *)ptr+strlen((char *)ptr),"/BitsPerComponent %lu def\r",nbit);
        sprintf((char *)ptr+strlen((char *)ptr),"/Decode [0 1] def\r");
        sprintf((char *)ptr+strlen((char *)ptr),"/DataSource {picstr1 readdata} def\r");
        sprintf((char *)ptr+strlen((char *)ptr),"currentdict end\r");
        sprintf((char *)ptr+strlen((char *)ptr),"%%%%BeginData: %llu ASCII Lines\r",(uint64_t)in->nlin+(uint64_t)1);
        sprintf((char *)ptr+strlen((char *)ptr),"beginimage\r");
    } else {
        sprintf((char *)ptr+strlen((char *)ptr),"gsave\r");
        sprintf((char *)ptr+strlen((char *)ptr),"/hascolor /statusdict where {pop\r");
        sprintf((char *)ptr+strlen((char *)ptr),"statusdict /processcolors known\r");
        sprintf((char *)ptr+strlen((char *)ptr),"{statusdict /processcolors get exec 1 gt}\r");
        sprintf((char *)ptr+strlen((char *)ptr),"{false} ifelse} {false} ifelse def\r");
        sprintf((char *)ptr+strlen((char *)ptr),"40 dict begin\r");
        sprintf((char *)ptr+strlen((char *)ptr),"/_image systemdict /image get def\r");
        sprintf((char *)ptr+strlen((char *)ptr),"/_setgray systemdict /setgray get def\r");
        sprintf((char *)ptr+strlen((char *)ptr),"/_currentgray systemdict /currentgray get def\r");
        sprintf((char *)ptr+strlen((char *)ptr),"/_settransfer systemdict /settransfer get def\r");
        sprintf((char *)ptr+strlen((char *)ptr),"/_currenttransfer systemdict /currenttransfer get def\r");
        sprintf((char *)ptr+strlen((char *)ptr),"/blank 0 _currenttransfer exec\r");
        sprintf((char *)ptr+strlen((char *)ptr),"1 _currenttransfer exec eq def\r");
        sprintf((char *)ptr+strlen((char *)ptr),"/negative blank\r");
        sprintf((char *)ptr+strlen((char *)ptr),"{0 _currenttransfer exec 0.5 lt}\r");
        sprintf((char *)ptr+strlen((char *)ptr),"{0 _currenttransfer exec 1 _currenttransfer exec gt}\r");
        sprintf((char *)ptr+strlen((char *)ptr),"ifelse def\r");
        sprintf((char *)ptr+strlen((char *)ptr),"/inverted? negative def\r");
        sprintf((char *)ptr+strlen((char *)ptr),"/level2 systemdict /languagelevel known\r");
        sprintf((char *)ptr+strlen((char *)ptr),"{languagelevel 2 ge} {false} ifelse def\r");
        sprintf((char *)ptr+strlen((char *)ptr),"/foureq {4 index eq 8 1 roll\r");
        sprintf((char *)ptr+strlen((char *)ptr),"4 index eq 8 1 roll\r");
        sprintf((char *)ptr+strlen((char *)ptr),"4 index eq 8 1 roll\r");
        sprintf((char *)ptr+strlen((char *)ptr),"4 index eq 8 1 roll\r");
        sprintf((char *)ptr+strlen((char *)ptr),"pop pop pop pop and and and} def\r");
        sprintf((char *)ptr+strlen((char *)ptr),"hascolor {/band 0 def} {/band 5 def} ifelse\r");
        sprintf((char *)ptr+strlen((char *)ptr),"/setcmykcolor where {pop\r");
        sprintf((char *)ptr+strlen((char *)ptr),"1 0 0 0 setcmykcolor _currentgray 1 exch sub\r");
        sprintf((char *)ptr+strlen((char *)ptr),"0 1 0 0 setcmykcolor _currentgray 1 exch sub\r");
        sprintf((char *)ptr+strlen((char *)ptr),"0 0 1 0 setcmykcolor _currentgray 1 exch sub\r");
        sprintf((char *)ptr+strlen((char *)ptr),"0 0 0 1 setcmykcolor _currentgray 1 exch sub\r");
        sprintf((char *)ptr+strlen((char *)ptr),"4 {4 copy} repeat\r");
        sprintf((char *)ptr+strlen((char *)ptr),"1 0 0 0 foureq {/band 1 store} if\r");
        sprintf((char *)ptr+strlen((char *)ptr),"0 1 0 0 foureq {/band 2 store} if\r");
        sprintf((char *)ptr+strlen((char *)ptr),"0 0 1 0 foureq {/band 3 store} if\r");
        sprintf((char *)ptr+strlen((char *)ptr),"0 0 0 1 foureq {/band 4 store} if\r");
        sprintf((char *)ptr+strlen((char *)ptr),"0 0 0 0 foureq {/band 6 store} if} if\r");
        sprintf((char *)ptr+strlen((char *)ptr),"blank {/band 6 store} if\r");
        sprintf((char *)ptr+strlen((char *)ptr),"gsave\r");
        sprintf((char *)ptr+strlen((char *)ptr),"/rows %lu def\r",in->nlin);
        sprintf((char *)ptr+strlen((char *)ptr),"/cols %lu def\r",in->npix);
        sprintf((char *)ptr+strlen((char *)ptr),"72 %llu mul %llu div dup cols mul exch rows mul scale\r",(uint64_t)in->nlin*(uint64_t)in->npix,(uint64_t)72*(uint64_t)in->nlin*(uint64_t)in->npix);
        sprintf((char *)ptr+strlen((char *)ptr),"level2 {\r");
        sprintf((char *)ptr+strlen((char *)ptr),"band 0 eq {\r");
        sprintf((char *)ptr+strlen((char *)ptr),"[/CIEBasedABC 3 dict begin\r");
        sprintf((char *)ptr+strlen((char *)ptr),"/DecodeLMN [{1.8 exp} bind dup dup] def\r");
        sprintf((char *)ptr+strlen((char *)ptr),"/MatrixLMN [\r");
        sprintf((char *)ptr+strlen((char *)ptr),"0.4497 0.2446 0.0252\r");
        sprintf((char *)ptr+strlen((char *)ptr),"0.3163 0.6720 0.1412\r");
        sprintf((char *)ptr+strlen((char *)ptr),"0.1845 0.0833 0.9227\r");
        sprintf((char *)ptr+strlen((char *)ptr),"] def\r");
        sprintf((char *)ptr+strlen((char *)ptr),"/WhitePoint [0.9505 1 1.0891] def\r");
        sprintf((char *)ptr+strlen((char *)ptr),"currentdict end]\r");
        sprintf((char *)ptr+strlen((char *)ptr),"} {/DeviceGray} ifelse\r");
        sprintf((char *)ptr+strlen((char *)ptr),"setcolorspace} if\r");
        sprintf((char *)ptr+strlen((char *)ptr),"/picstr1 %lu string def\r",in->npix);
        sprintf((char *)ptr+strlen((char *)ptr),"/picstr2 %lu string def\r",in->npix);
        sprintf((char *)ptr+strlen((char *)ptr),"/picstr3 %lu string def\r",in->npix);
        sprintf((char *)ptr+strlen((char *)ptr),"/picstr4 %lu string def\r",in->npix);
        sprintf((char *)ptr+strlen((char *)ptr),"/readdata {currentfile exch readhexstring pop} def\r");
        sprintf((char *)ptr+strlen((char *)ptr),"/image2 level2 {/image load def} {{begin\r");
        sprintf((char *)ptr+strlen((char *)ptr),"Width Height BitsPerComponent ImageMatrix\r");
        sprintf((char *)ptr+strlen((char *)ptr),"Decode length 2 eq\r");
        sprintf((char *)ptr+strlen((char *)ptr),"{/DataSource load image} if\r");
        sprintf((char *)ptr+strlen((char *)ptr),"Decode length 6 eq\r");
        sprintf((char *)ptr+strlen((char *)ptr),"{DataSource 0 get DataSource 1 get DataSource 2 get\r");
        sprintf((char *)ptr+strlen((char *)ptr),"true 3 colorimage} if\r");
        sprintf((char *)ptr+strlen((char *)ptr),"Decode length 8 eq\r");
        sprintf((char *)ptr+strlen((char *)ptr),"{DataSource 0 get DataSource 1 get\r");
        sprintf((char *)ptr+strlen((char *)ptr),"DataSource 2 get DataSource 3 get\r");
        sprintf((char *)ptr+strlen((char *)ptr),"true 4 colorimage} if\r");
        sprintf((char *)ptr+strlen((char *)ptr),"end} def} ifelse\r");
        sprintf((char *)ptr+strlen((char *)ptr),"/_image2 level2 {/_image load def} {{begin\r");
        sprintf((char *)ptr+strlen((char *)ptr),"Width Height BitsPerComponent ImageMatrix\r");
        sprintf((char *)ptr+strlen((char *)ptr),"/DataSource load _image end} def} ifelse\r");
        sprintf((char *)ptr+strlen((char *)ptr),"/beginimage {\r");
        sprintf((char *)ptr+strlen((char *)ptr),"band 0 eq band 4 eq or band 5 eq or\r");
        sprintf((char *)ptr+strlen((char *)ptr),"{image2}\r");
        sprintf((char *)ptr+strlen((char *)ptr),"{negative {{pop 0}} {{pop 1}} ifelse\r");
        sprintf((char *)ptr+strlen((char *)ptr),"_settransfer _image2} ifelse\r");
        sprintf((char *)ptr+strlen((char *)ptr),"} def\r");
        sprintf((char *)ptr+strlen((char *)ptr),"12 dict begin\r");
        sprintf((char *)ptr+strlen((char *)ptr),"/ImageType 1 def\r");
        sprintf((char *)ptr+strlen((char *)ptr),"/Width cols def\r");
        sprintf((char *)ptr+strlen((char *)ptr),"/Height rows def\r");
        sprintf((char *)ptr+strlen((char *)ptr),"/ImageMatrix [cols 0 0 rows neg 0 rows] def\r");
        sprintf((char *)ptr+strlen((char *)ptr),"/BitsPerComponent %lu def\r",nbit);
        sprintf((char *)ptr+strlen((char *)ptr),"band 0 eq\r");
        sprintf((char *)ptr+strlen((char *)ptr),"{/Decode [0 1 0 1 0 1] def\r");
        sprintf((char *)ptr+strlen((char *)ptr),"/MultipleDataSources true def\r");
        sprintf((char *)ptr+strlen((char *)ptr),"/DataSource [\r");
        sprintf((char *)ptr+strlen((char *)ptr),"{picstr1 readdata}\r");
        sprintf((char *)ptr+strlen((char *)ptr),"{picstr2 readdata}\r");
        sprintf((char *)ptr+strlen((char *)ptr),"{picstr3 readdata picstr4 readdata pop}\r");
        sprintf((char *)ptr+strlen((char *)ptr),"] def}\r");
        sprintf((char *)ptr+strlen((char *)ptr),"{/Decode [0 1] def\r");
        sprintf((char *)ptr+strlen((char *)ptr),"/DataSource {\r");
        sprintf((char *)ptr+strlen((char *)ptr),"picstr1 readdata pop\r");
        sprintf((char *)ptr+strlen((char *)ptr),"picstr2 readdata pop\r");
        sprintf((char *)ptr+strlen((char *)ptr),"picstr3 readdata pop\r");
        sprintf((char *)ptr+strlen((char *)ptr),"picstr4 readdata\r");
        sprintf((char *)ptr+strlen((char *)ptr),"} def}\r");
        sprintf((char *)ptr+strlen((char *)ptr),"ifelse\r");
        sprintf((char *)ptr+strlen((char *)ptr),"currentdict end\r");
        sprintf((char *)ptr+strlen((char *)ptr),"%%%%BeginData: %llu ASCII Lines\r",(uint64_t)4*(uint64_t)in->nlin+(uint64_t)1);
        sprintf((char *)ptr+strlen((char *)ptr),"beginimage\r");
    }
#if defined SADIE_UNIX
    total = (uint64_t)strlen((char *)ptr);
    if (fwrite(ptr,(size_t)total,1,fp) != 1) {
        sprintf(msg," Can't write image header to file %s.",name);
        MESSAGE('E',msg);
        goto the_end;
    }      
#elif defined SADIE_MACOS
    nbytes = (long)strlen((char *)ptr);
    if ((error=FSWrite(ref,&nbytes,ptr)) != noErr) {
        sprintf(msg," Can't write image header to file %s.\rFile Manager Error Code: %d.",name,error);
        MESSAGE('E',msg);
        goto the_end;
    }
#endif

    /* write image data to file */
    factor = 255.0 / (in->gmax-in->gmin);
    total  = (uint64_t)2*(uint64_t)in->npix + (uint64_t)1;
    nbytes = (long)2*(long)in->npix + (long)1;
    ptr[(uint64_t)2*(uint64_t)in->npix] = '\r';
    if (in->nbnd != 3) {
        for (j=0; j<in->nlin; j+=1) {
            for (ptr=data, k=0; k<in->npix; k+=1, ptr+=2) {
                dval   = (in->data[0][j][k]-in->gmin)*factor;
                cval   = (unsigned char)dval;
                ptr[0] = hex[(cval & 0xF0)  >>  4];
                ptr[1] = hex[(cval & 0x0F)];
            }
#if defined SADIE_UNIX
            if (fwrite(data,(size_t)total,1,fp) != 1) {
                sprintf(msg," Can't write image data to file %s.",name);
                MESSAGE('E',msg);
                goto the_end;
            }
#elif defined SADIE_MACOS
            if ((error=FSWrite(ref,&nbytes,data)) != noErr) {
                sprintf(msg," Can't write image data to file %s.\rFile Manager Error Code: %d.",name,error);
                MESSAGE('E',msg);
                goto the_end;
            }
#endif
        }
    } else {
        for (j=0; j<in->nlin; j+=1) {
            for (i=0; i<in->nbnd; i+=1) {
                for (ptr=data, k=0; k<in->npix; k+=1, ptr+=2) {
                    dval   = (in->data[i][j][k]-in->gmin)*factor;
                    cval   = (unsigned char)dval;
                    ptr[0] = hex[(cval & 0xF0)  >>  4];
                    ptr[1] = hex[(cval & 0x0F)];
                }
#if defined SADIE_UNIX
                if (fwrite(data,(size_t)total,1,fp) != 1) {
                    sprintf(msg," Can't write image data to file %s.",name);
                    MESSAGE('E',msg);
                    goto the_end;
                }
#elif defined SADIE_MACOS
                if ((error=FSWrite(ref,&nbytes,data)) != noErr) {
                    sprintf(msg," Can't write image data to file %s.\rFile Manager Error Code: %d.",name,error);
                    MESSAGE('E',msg);
                    goto the_end;
                }
#endif
            }
            for (ptr=data, k=0; k<in->npix; k+=1, ptr+=2) {
                dval   = (in->data[0][j][k]-in->gmin)*factor*0.30 + (in->data[1][j][k]-in->gmin)*factor*0.59 + (in->data[2][j][k]-in->gmin)*factor*0.11;
                cval   = (unsigned char)rnd(dval);
                ptr[0] = hex[(cval & 0xF0)  >>  4];
                ptr[1] = hex[(cval & 0x0F)];
            }
#if defined SADIE_UNIX
            if (fwrite(data,(size_t)total,1,fp) != 1) {
                sprintf(msg," Can't write image data to file %s.",name);
                MESSAGE('E',msg);
                goto the_end;
            }
#elif defined SADIE_MACOS
            if ((error=FSWrite(ref,&nbytes,data)) != noErr) {
                sprintf(msg," Can't write image data to file %s.\rFile Manager Error Code: %d.",name,error);
                MESSAGE('E',msg);
                goto the_end;
            }
#endif
        }
    }

    /* write image trailer to file */
    sprintf((char *)ptr                    ,"%%%%EndData\r");
    sprintf((char *)ptr+strlen((char *)ptr),"grestore end\r");
    sprintf((char *)ptr+strlen((char *)ptr),"grestore\r");
#if defined SADIE_UNIX
    total = (uint64_t)strlen((char *)ptr);
    if (fwrite(ptr,(size_t)total,1,fp) != 1) {
        sprintf(msg," Can't write image trailer to file %s.",name);
        MESSAGE('E',msg);
        goto the_end;
    }
#elif defined SADIE_MACOS
    nbytes = (long)strlen((char *)ptr);
    if ((error=FSWrite(ref,&nbytes,ptr)) != noErr) {
        sprintf(msg," Can't write image trailer to file %s.\rFile Manager Error Code: %d.",name,error);
        MESSAGE('E',msg);
        goto the_end;
    }
    c2pstrcpy(pstr,(char *)name);
    if ((error=FlushVol(pstr,vref)) != noErr  ||  (error=GetFPos(ref,&nbytes)) != noErr  ||  (error=SetEOF(ref,nbytes)) != noErr) {
        sprintf(msg," Can't write image trailer to file %s.\rFile Manager Error Code: %d.",name,error);
        MESSAGE('E',msg);
        goto the_end;
    }
#endif

    /* close image file */
    the_end:
#if defined SADIE_UNIX
    if (fp) {
        if (fclose(fp)) {
            sprintf(msg," Can't close file %s.",name);
            MESSAGE('W',msg);
        }
    }
#elif defined SADIE_MACOS
    if ((error=FSClose(ref)) != noErr) {
        sprintf(msg," Can't close file %s.\rFile Manager Error Code: %d.",name,error);
        MESSAGE('W',msg);
    }
#endif
    if (data)  free(data);
    if (TIMES) TIMING(T_EXIT);
}
