#if HAVE_CONFIG_H
#include <config.h>
#endif /* HAVE_CONFIG_H */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif /* HAVE_CONFIG_H */
#include "sadie.h"

#if defined SADIE_MACOS
#include <TextUtils.h>
#include <Files.h>
#endif

#define                TBYTE                            ((unsigned long)1)
#define                TASCII                           ((unsigned long)2)
#define                TSHORT                           ((unsigned long)3)
#define                TLONG                            ((unsigned long)4)
#define                TRATIONAL                        ((unsigned long)5)

#define                LilEndian                        ((unsigned long)0x4949)
#define                BigEndian                        ((unsigned long)0x4D4D)
#define                MagicNumber                      ((unsigned long)42)

#define                ImageWidth                       256
#define                ImageLength                      257
#define                BitsPerSample                    258
#define                Compression                      259

#define                PhotometricInterpretation        262

#define                StripOffsets                     273

#define                SamplesPerPixel                  277
#define                RowsPerStrip                     278
#define                StripByteCounts                  279

#define                XResolution                      282
#define                YResolution                      283

#define                ResolutionUnit                   296

typedef struct {
    unsigned short tag;
    unsigned short type;
    unsigned long  count;
    unsigned long  value;
} entry;

static void SetEntry (e, tag, type, count, value) register entry *e; unsigned short tag; unsigned long type; unsigned long count; unsigned long value; {

    e->tag   = tag;
    e->type  = (unsigned short)type;
    e->count = count;
    if (type == TSHORT  &&  count == 1) {
        *(unsigned short *)&(e->value) = (unsigned short)value;
    } else {
        e->value = value;
    }
}

/*-Copyright Information------------------------------------------------------*/
/* Copyright (c) 1993 by the University of Arizona Digital Image Analysis Lab */
/*----------------------------------------------------------------------------*/
/*-General Information--------------------------------------------------------*/
/*                                                                            */
/*   This function writes a TIFF image to disk.                               */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/*-Interface Information------------------------------------------------------*/
void IMG2TIFF (
IMAGE *in,           /*  I   Pointer to the image.                            */
unsigned char *name, /*  I   String, containing the name of the disk file.    */
uint32_t nbit,       /*  I   Bits/pixel in disk file.                         */
short ref,           /*  I   Reference number for the disk file.              */
short vref           /*  I   Volume reference number for the disk file.       */
/*----------------------------------------------------------------------------*/
) { static char array[2] = { 0x00, 0x01 };
    register uint32_t i, j, k;
    uint64_t total;
    unsigned char *data, *ptr;
    unsigned long addr[5];
    char   msg[SLEN];
    short  error;
    long   nbytes;
    double factor;
#if defined SADIE_UNIX
    FILE   *fp = NULL;
#elif defined SADIE_MACOS
    Str255 pstr;
#endif

    if (TIMES) TIMING(T_IMG2TIFF);
    if (NAMES) {
        MESSAGE('I',"");
        MESSAGE('I',"IMG2TIFF");
    }

    /* check input */
    if (!CHECKIMG(in)) {
        MESSAGE('E'," Can't identify image.");
        goto the_end;
    }

    if (NAMES) {
        MESSAGE('I',"");
        sprintf(msg," Input image:               %s",in->text);
        MESSAGE('I',msg);
        sprintf(msg," Output disk file:          %s",name);
        MESSAGE('I',msg);
        sprintf(msg," Output format:             %d-bit TIFF",nbit);
        MESSAGE('I',msg);
        MESSAGE('I'," ...............");
    }

#if defined SADIE_UNIX
    /* open image file */
    if (!(fp=fopen(name,"w"))) {
        sprintf(msg," Can't open file %s.",name);
        MESSAGE('E',msg);
        goto the_end;
    }
#endif

    /* write image header to file */
    addr[0] = sizeof(unsigned short) + sizeof(unsigned short) + sizeof(unsigned long) + sizeof(unsigned short) + 12*sizeof(entry) + sizeof(unsigned long);
    addr[1] = sizeof(unsigned short) + sizeof(unsigned short) + sizeof(unsigned long) + sizeof(unsigned short) + 12*sizeof(entry) + sizeof(unsigned long) + 3*sizeof(unsigned short);
    addr[2] = sizeof(unsigned short) + sizeof(unsigned short) + sizeof(unsigned long) + sizeof(unsigned short) + 12*sizeof(entry) + sizeof(unsigned long) + 3*sizeof(unsigned short) +   in->nlin*sizeof(unsigned long);
    addr[3] = sizeof(unsigned short) + sizeof(unsigned short) + sizeof(unsigned long) + sizeof(unsigned short) + 12*sizeof(entry) + sizeof(unsigned long) + 3*sizeof(unsigned short) + 2*in->nlin*sizeof(unsigned long);
    addr[4] = sizeof(unsigned short) + sizeof(unsigned short) + sizeof(unsigned long) + sizeof(unsigned short) + 12*sizeof(entry) + sizeof(unsigned long) + 3*sizeof(unsigned short) + 2*in->nlin*sizeof(unsigned long) + 2*sizeof(unsigned long);
    total   = ((uint64_t)sizeof(unsigned short) + (uint64_t)sizeof(unsigned short) + (uint64_t)sizeof(unsigned long) + (uint64_t)sizeof(unsigned short) + (uint64_t)12*(uint64_t)sizeof(entry) + (uint64_t)sizeof(unsigned long) + (uint64_t)3*(uint64_t)sizeof(unsigned short) + (uint64_t)2*(uint64_t)in->nlin*(uint64_t)sizeof(unsigned long) + (uint64_t)4*(uint64_t)sizeof(unsigned long)) * (uint64_t)sizeof(unsigned char);
    nbytes  = (long)total;
    if (total > (uint64_t)SIZE_MAX  ||  (ptr=data=(unsigned char *)malloc((size_t)total)) == NULL) {
        MESSAGE('E'," Memory request failed.");
        goto the_end;
    }
    *(unsigned short *)ptr = *(short *)array != 1 ? LilEndian : BigEndian;                                   ptr += sizeof(unsigned short);
    *(unsigned short *)ptr = MagicNumber;                                                                    ptr += sizeof(unsigned short);
    *(unsigned long  *)ptr = 8ul;                                                                            ptr += sizeof(unsigned long);
    *(unsigned short *)ptr = 12u;                                                                            ptr += sizeof(unsigned short);
    SetEntry(ptr,ImageWidth               ,TLONG    ,1ul,(unsigned long)in->npix);                           ptr += sizeof(entry);
    SetEntry(ptr,ImageLength              ,TLONG    ,1ul,(unsigned long)in->nlin);                           ptr += sizeof(entry);
    SetEntry(ptr,BitsPerSample            ,TSHORT   ,(unsigned long)in->nbnd,in->nbnd == 1 ? 8ul : addr[0]); ptr += sizeof(entry);
    SetEntry(ptr,Compression              ,TSHORT   ,1ul,1ul);                                               ptr += sizeof(entry);
    SetEntry(ptr,PhotometricInterpretation,TSHORT   ,1ul,in->nbnd == 1 ? 1ul : 2ul);                         ptr += sizeof(entry);
    SetEntry(ptr,StripOffsets             ,TLONG    ,(unsigned long)in->nlin,addr[1]);                       ptr += sizeof(entry);
    SetEntry(ptr,SamplesPerPixel          ,TSHORT   ,1ul,(unsigned long)in->nbnd);                           ptr += sizeof(entry);
    SetEntry(ptr,RowsPerStrip             ,TLONG    ,1ul,1ul);                                               ptr += sizeof(entry);
    SetEntry(ptr,StripByteCounts          ,TLONG    ,(unsigned long)in->nlin,addr[2]);                       ptr += sizeof(entry);
    SetEntry(ptr,XResolution              ,TRATIONAL,1ul,addr[3]);                                           ptr += sizeof(entry);
    SetEntry(ptr,YResolution              ,TRATIONAL,1ul,addr[4]);                                           ptr += sizeof(entry);
    SetEntry(ptr,ResolutionUnit           ,TSHORT   ,1ul,2ul);                                               ptr += sizeof(entry);
    *(unsigned long  *)ptr = 0ul;                                                                            ptr += sizeof(unsigned long);
    *(unsigned short *)ptr = (unsigned short)nbit;                                                           ptr += sizeof(unsigned short);
    *(unsigned short *)ptr = (unsigned short)nbit;                                                           ptr += sizeof(unsigned short);
    *(unsigned short *)ptr = (unsigned short)nbit;                                                           ptr += sizeof(unsigned short);
    for (i=0; i<in->nlin; i++) {
        *(unsigned long *)ptr = (unsigned long)(total + (uint64_t)i*(uint64_t)in->nbnd*(uint64_t)in->npix);  ptr += sizeof(unsigned long);
    }
    for (i=0; i<in->nlin; i++) {
        *(unsigned long *)ptr = (unsigned long)((uint64_t)in->nbnd*(uint64_t)in->npix);                      ptr += sizeof(unsigned long);
    }
    *(unsigned long  *)ptr = 72ul;                                                                           ptr += sizeof(unsigned long);
    *(unsigned long  *)ptr =  1ul;                                                                           ptr += sizeof(unsigned long);
    *(unsigned long  *)ptr = 72ul;                                                                           ptr += sizeof(unsigned long);
    *(unsigned long  *)ptr =  1ul;                                                                           ptr += sizeof(unsigned long);
#if defined SADIE_UNIX
    if (fwrite(data,(size_t)total,1,fp) != 1) {
        sprintf(msg," Can't write image header to file %s.",name);
        MESSAGE('E',msg);
        goto the_end;
    }    
#elif defined SADIE_MACOS
    if ((error=FSWrite(ref,&nbytes,data)) != noErr) {
        sprintf(msg," Can't write image header to file %s.\rFile Manager Error Code: %d.",name,error);
        MESSAGE('E',msg);
        goto the_end;
    }
#endif
    if (data) free(data);

    /* write image data to file */
    factor = 255.0 / (in->gmax-in->gmin);
    total  = (uint64_t)in->nbnd*(uint64_t)in->npix*(uint64_t)sizeof(unsigned char);
    nbytes = (long)total;
    if (total > (uint64_t)SIZE_MAX  ||  (ptr=data=(unsigned char *)malloc((size_t)total)) == NULL) {
        MESSAGE('E'," Memory request failed.");
        goto the_end;
    }
    if (in->nbnd == 1) {
        for (j=0; j<in->nlin; j++) {
            for (ptr=data, k=0; k<in->npix; k+=1, ptr+=1) {
                ptr[0] = (unsigned char)((in->data[0][j][k]-in->gmin)*factor);
            }
#if defined SADIE_UNIX
            if (fwrite(data,(size_t)total,1,fp) != 1) {
                sprintf(msg," Can't write image data to file %s.",name);
                MESSAGE('E',msg);
                goto the_end;
            }
#elif defined SADIE_MACOS
            if ((error=FSWrite(ref,&nbytes,data)) != noErr) {
                sprintf(msg," Can't write image data to file %s.\rFile Manager Error Code: %d.",name,error);
                MESSAGE('E',msg);
                goto the_end;
            }
#endif
        }
    } else {
        for (j=0; j<in->nlin; j++) {
            for (ptr=data, k=0; k<in->npix; k+=1, ptr+=3) {
                ptr[0] = (unsigned char)((in->data[0][j][k]-in->gmin)*factor);
                ptr[1] = (unsigned char)((in->data[1][j][k]-in->gmin)*factor);
                ptr[2] = (unsigned char)((in->data[2][j][k]-in->gmin)*factor);
            }
#if defined SADIE_UNIX
            if (fwrite(data,(size_t)total,1,fp) != 1) {
                sprintf(msg," Can't write image data to file %s.",name);
                MESSAGE('E',msg);
                goto the_end;
            }
#elif defined SADIE_MACOS
            if ((error=FSWrite(ref,&nbytes,data)) != noErr) {
                sprintf(msg," Can't write image data to file %s.\rFile Manager Error Code: %d.",name,error);
                MESSAGE('E',msg);
                goto the_end;
            }
#endif
        }
    }
#if defined SADIE_MACOS
    c2pstrcpy(pstr,(char *)name);
    if ((error=FlushVol(pstr,vref)) != noErr  ||  (error=GetFPos(ref,&nbytes)) != noErr  ||  (error=SetEOF(ref,nbytes)) != noErr) {
        sprintf(msg," Can't write image data to file %s.\rFile Manager Error Code: %d.",name,error);
        MESSAGE('E',msg);
        goto the_end;
    }
#endif

    /* close image file */
    the_end:
#if defined SADIE_UNIX
    if (fp) {
        if (fclose(fp)) {
            sprintf(msg," Can't close file %s.",name);
            MESSAGE('W',msg);
        }
    }
#elif defined SADIE_MACOS
    if ((error=FSClose(ref)) != noErr) {
        sprintf(msg," Can't close file %s.\rFile Manager Error Code: %d.",name,error);
        MESSAGE('W',msg);
    }
#endif
    if (data)  free(data);
    if (TIMES) TIMING(T_EXIT);
}
