#ifdef HAVE_CONFIG_H
#include <config.h>
#endif /* HAVE_CONFIG_H */
#include   "sadie.h"

#if defined SADIE_LIBJPEG

#include   "jpeglib.h"

struct my_error_mgr {
  struct jpeg_error_mgr pub;
  jmp_buf setjmp_buffer;
};

typedef struct my_error_mgr * my_error_ptr;

METHODDEF(void) my_output_message (j_common_ptr cinfo) {
  char buffer[JMSG_LENGTH_MAX];

  /* Create and display the message */
  buffer[0] = ' ';
  (*cinfo->err->format_message) (cinfo, buffer+1);
  MESSAGE('W',buffer);
}

METHODDEF(void) my_error_exit (j_common_ptr cinfo) {

  /* cinfo->err really points to a my_error_mgr struct, so coerce pointer */
  my_error_ptr myerr = (my_error_ptr)cinfo->err;

  /* Always display the message. */
  /* We could postpone this until after returning, if we chose. */
  (*cinfo->err->output_message)(cinfo);

  /* Return control to the setjmp point */
  longjmp(myerr->setjmp_buffer, 1);
}

/*-Copyright Information------------------------------------------------------*/
/* Copyright (c) 2001 by the University of Arizona Digital Image Analysis Lab */
/*----------------------------------------------------------------------------*/
/*-General Information--------------------------------------------------------*/
/*                                                                            */
/*   This function reads a JPEG (JFIF) image from disk into main memory.      */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/*-Interface Information------------------------------------------------------*/
void JPEG2IMG (
unsigned char *name, /*  I   String, containing the name of the disk file.    */
short ref,           /*  I   Reference number of the disk file.               */
IMAGE **out          /*  O   Address of a pointer to the image.               */
/*----------------------------------------------------------------------------*/
) { register uint32_t i, j, k;
    char msg[SLEN];
    struct jpeg_decompress_struct cinfo;
    struct my_error_mgr jerr;
    JSAMPARRAY buffer;
    JSAMPLE *pixel;

    if (TIMES) TIMING(T_JPEG2IMG);

    /* set up standard JPEG error routines, then override error_exit */
    cinfo.err = jpeg_std_error(&jerr.pub);
    jerr.pub.output_message = my_output_message;
    jerr.pub.error_exit = my_error_exit;
    if (setjmp(jerr.setjmp_buffer) == 0) {

        /* set up data structures and read header data */
        jpeg_create_decompress(&cinfo);
        jpeg_stdio_src(&cinfo,ref);
        jpeg_read_header(&cinfo,TRUE);
        jpeg_start_decompress(&cinfo);

        /* create image of appropriate size */
        if (!CHECKIMG(*out)) GETMEM((uint32_t)cinfo.num_components,(uint32_t)cinfo.image_height,(uint32_t)cinfo.image_width,out);
        if (!*out) goto the_end;

        /* read image data */
        buffer = (*cinfo.mem->alloc_sarray)((j_common_ptr)&cinfo,JPOOL_IMAGE,cinfo.output_width*cinfo.output_components,1);
        for (j=0; cinfo.output_scanline < cinfo.output_height; j++) {
            jpeg_read_scanlines(&cinfo,buffer,1);
            pixel = buffer[0];
            for (k=0; k<(*out)->npix; k++) {
                for (i=0; i<(*out)->nbnd; i++) {
                    (*out)->data[i][j][k] = (PIXEL)(*pixel);
                    pixel++;
                }
            }
        }
        jpeg_finish_decompress(&cinfo);

        if (NAMES) {
            RANGE(*out);
            MESSAGE('I',"");
            MESSAGE('I',"JPEG2IMG");
            MESSAGE('I',"");
            sprintf(msg," Input file:                                 %s",name);
            MESSAGE('I',msg);
            if (cinfo.jpeg_color_space == JCS_UNKNOWN) {
                sprintf(msg," Color space:                                Unknown");
            } else if (cinfo.jpeg_color_space == JCS_GRAYSCALE) {
                sprintf(msg," Color space:                                Grayscale");
            } else if (cinfo.jpeg_color_space == JCS_RGB) {
                sprintf(msg," Color space:                                RGB");
            } else if (cinfo.jpeg_color_space == JCS_YCbCr) {
                sprintf(msg," Color space:                                YCbCr");
            } else if (cinfo.jpeg_color_space == JCS_CMYK) {
                sprintf(msg," Color space:                                CMYK");
            } else if (cinfo.jpeg_color_space == JCS_YCCK) {
                sprintf(msg," Color space:                                YCbCrK");
            }
            MESSAGE('I',msg); 
            sprintf(msg," Number of bands, lines, pixels:             %lu, %lu, %lu",(*out)->nbnd,(*out)->nlin,(*out)->npix);
            MESSAGE('I',msg);
            sprintf(msg," Number of bits/pixel/band (JPEG file):      %lu",(uint32_t)cinfo.data_precision);
            MESSAGE('I',msg);
            sprintf(msg," Graylevel minimum, maximum:                 %10.4e, %10.4e",(*out)->gmin,(*out)->gmax);
            MESSAGE('I',msg);
            MESSAGE('I'," ...............");
        }

        the_end:
        jpeg_destroy_decompress(&cinfo);
    }

    if (TIMES) TIMING(T_EXIT);
}
#endif
