#ifdef HAVE_CONFIG_H
#include <config.h>
#endif /* HAVE_CONFIG_H */
#include        "sadie.h"

/*-Copyright Information------------------------------------------------------*/
/* Copyright (c) 1994 by the University of Arizona Digital Image Analysis Lab */
/*----------------------------------------------------------------------------*/
/*-General Information--------------------------------------------------------*/
/*                                                                            */
/*   This function performs a maximum likelihood classification on an image.  */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/*-Interface Information------------------------------------------------------*/
void MAXLIKE (
IMAGE  *in,    /*  I  Pointer to the input image.                             */
uint32_t nlev, /*  I  Number of classes                                       */
               /*     (and number of graylevels in the output image).         */
double *cov,   /*  I  Pointer to the array                                    */
               /*     of covariance matrices[nlev][nbnd][nbnd].               */
double *prob,  /*  I  Pointer to the array[nlev] of a priori probabilities.   */
IMAGE  **out   /*  O  Address of a pointer to the output image.               */
/*--------------------------------------------------------------------------- */
) { register uint64_t i, j, k;                              
    char msg[SLEN];
    short m;

    if (TIMES) TIMING(T_MAXLIKE);
    if (NAMES) {
        MESSAGE('I',"");
        MESSAGE('I',"MAXLIKE");
    }

    /* check input */
    if (!CHECKIMG(in)) {
        MESSAGE('E'," Can't identify input image.");
        goto the_end;
    }

    if (NAMES) {
        MESSAGE('I',"");
        sprintf(msg," Input image:          %s",in->text);
        MESSAGE('I',msg);
        sprintf(msg," Number of classes:    %lu",nlev);
        MESSAGE('I',msg);
        for (i=0; i<nlev; i++) {
            sprintf(msg," Class %llu:          ",i+1);
            MESSAGE('I',msg);
            MESSAGE('I',"  Covariance matrix:");
            for (j=0; j<min(3,in->nbnd); j++) {
                sprintf(msg,"  Band %6llu:       ",j+1);
                for (k=0; k<min(3,in->nbnd); k++) {
                    sprintf(msg+strlen(msg),"%12.4e",cov[i*in->nbnd*in->nbnd+j*in->nbnd+k]);
                }
                MESSAGE('I',msg);
            }
            sprintf(msg,"  Probability:       %12.4e",prob[i]);
            MESSAGE('I',msg);
        }
        MESSAGE('I'," ...............");
    }

    (*out) = NULL;

    the_end:
    if (TIMES) TIMING(T_EXIT);
}
