#if defined SADIE_IPT
#include "IPT.h"    
#else
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif /* HAVE_CONFIG_H */
#include "sadie.h"
#ifdef SADIE_LIB
#include "app_sadie.h"
#endif /* defined SADIE_LIB */
#endif /* !defined SADIE_IPT */

#define AXIS    12

#if defined SADIE_LIB

/* Application-provided plotting callback */
static app_plotter_t app_plotter;

/*-Copyright Information------------------------------------------------------*/
/* Copyright (c) 2004 by the University of Arizona Digital Image Analysis Lab */
/*----------------------------------------------------------------------------*/
/*-General Information--------------------------------------------------------*/
/*                                                                            */
/*   This registers a function provided by the application using the SADIE    */
/*   library to plot a histogram, returning zero or an error code.            */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/*-Interface Information------------------------------------------------------*/
int sad_register_plotter_callback (
app_plotter_t thecallback  /*  I  Histogram plotting function callback. */
/*----------------------------------------------------------------------------*/
) {
    if (thecallback == NULL) return -1;
    app_plotter = thecallback;
    return 0;
}

#endif /* defined SADIE_LIB */

/*-Copyright Information------------------------------------------------------*/
/* Copyright (c) 1988 by the University of Arizona Digital Image Analysis Lab */
/*----------------------------------------------------------------------------*/
/*-General Information--------------------------------------------------------*/
/*                                                                            */
/*   This function outputs a graphic of a series of data arrays (lines).      */
/*   The ordinate is plotted horizontally, and the abscissa vertically.       */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/*-Interface Information------------------------------------------------------*/
short PLOT (
PIXEL *line,    /*  I   Pointer to an array[nbnd*npix] of pixel values.       */
uint32_t nbnd,  /*  I   Number of lines to plot.                              */
uint32_t npix,  /*  I   Number of pixels per line.                            */
char *minxlbl,  /*  I   Minimum value to print on x-axis.                     */
char *maxxlbl,  /*  I   Maximum value to print on x-axis.                     */
short option    /*  I   Switch, indicating the type of output:                */
                /*      UNIT   -   Ordinate range is [-1,+1].                 */
                /*      NORM   -   Normalized ordinate.                       */
                /*      LOG    -   Logarithmic ordinate.                      */
/*----------------------------------------------------------------------------*/
) { uint64_t i;
    PIXEL gmin, gmax;
    char  msg[SLEN];
#ifndef SADIE_LIB
    register uint64_t j, npts;
    short height;
#if defined SADIE_IPT
    GrafPtr port;
    Rect rect;
    PicHandle p;
    Str255 pstr;
#else  /* !defined SADIE_IPT */
    static char *axis[] = { "           +", "-----------+" };
#endif /* !defined SADIE_IPT */
#endif /* !defined SADIE_LIB */

    if (TIMES) TIMING(T_PLOT);
    if (NAMES) {
        MESSAGE('I',"");
        MESSAGE('I',"PLOT");
    }

    /* check input */
    if (option != UNIT  &&  option != NORM  &&  option != LOG) {
        MESSAGE('E'," Option must be one of UNIT, NORM, or LOG.");
        goto the_end;
    }

    if (NAMES) {
        MESSAGE('I',"");
        sprintf(msg," Number of bands:    %lu",(unsigned long)nbnd);
        MESSAGE('I',msg);
        sprintf(msg," Number of pixels:   %lu",(unsigned long)npix);
        MESSAGE('I',msg);
        if (option == UNIT) {
            MESSAGE('I'," Ordinate range is [-1,+1]");
        } else if (option == NORM) {
            MESSAGE('I'," Normalized ordinate");
        } else if (option == LOG) {
            MESSAGE('I'," Logarithmic ordinate");
        }
        MESSAGE('I'," ...............");
    }

    /* find minimum and maximum */
    if (option == UNIT) {
        gmin = (PIXEL)-1;
        gmax = (PIXEL) 1;
    } else {
        gmax = gmin = line[0];
        for (i=1; i<(uint64_t)nbnd*(uint64_t)npix; i++) {
            gmin = min(gmin,line[i]);
            gmax = max(gmax,line[i]);
        }
        if (option == LOG) {
            if (gmin < (PIXEL)0) {
                MESSAGE('E'," Graylevel values must be greater than or equal to zero for logarithmic ordinate.");
                goto the_end;
            }
            gmin = (PIXEL)log10((double)gmin+1.0);
            gmax = (PIXEL)log10((double)gmax+1.0);
        }
    }

#if defined SADIE_LIB
    (*app_plotter)(line,nbnd,npix,minxlbl,maxxlbl,option,gmin,gmax,app_data);
#elif defined SADIE_IPT
    GetPort(&port);
    SetPortWindowPort(gSessionLog->window);
    SetRect(&rect,0,0,nbnd*(npix+70),102);
    p = OpenPicture(&rect);
    for (i=0; i<nbnd; i++) {

        /* check for empty histogram */
        for (npts=j=0; j<npix; j++) {
            npts += line[i*npix+j];
        }
        if (npts == 0) {
            sprintf(msg," Histogram for band %llu is empty.",i+1);
            MESSAGE('E',msg);
            continue;
        }

        if (nbnd != 3) {
            ForeColor(blackColor);
        } else if (i == 0) {
            ForeColor(redColor);
        } else if (i == 1) {
            ForeColor(greenColor);
        } else {
            ForeColor(blueColor);
        }

        /* draw and label axes */
        MoveTo(i*(npix+70)+35,10);
        LineTo(i*(npix+70)+39,10);
        LineTo(i*(npix+70)+39,83);
        MoveTo(i*(npix+70)+35,83);
        LineTo(i*(npix+70)+39+npix,83);
        for (j=0; j<=npix; j++) {
            if (j%16 == 0) {
                MoveTo(i*(npix+70)+j+39,83);
                LineTo(i*(npix+70)+j+39,83+(j%32 ? 2:4));
            }
        }
        sprintf(msg,"%d",rnd(gmax));
        c2pstrcpy(pstr,msg);
        MoveTo(i*(npix+70)+35-StringWidth(pstr),15);
        DrawString(pstr);
        sprintf(msg,"%d",rnd(gmin));
        c2pstrcpy(pstr,msg);
        MoveTo(i*(npix+70)+35-StringWidth(pstr),87);
        DrawString(pstr);
        c2pstrcpy(pstr,minxlbl);
        MoveTo(i*(npix+70)+39-StringWidth(pstr)/2,97);
        DrawString(pstr);
        c2pstrcpy(pstr,maxxlbl);
        MoveTo(i*(npix+70)+39-StringWidth(pstr)/2+npix,97);
        DrawString(pstr);

        /* plot the line data */
        for (j=0; j<npix; j++) {
            if (option == UNIT) {
                height = max(0,min(rnd(60.0*(double)(line[i*npix+j]-gmin)/(double)(gmax-gmin)),72));
            } else if (option == NORM) {
                height = rnd(72.0*(double)(line[i*npix+j]-gmin)/(double)(gmax-gmin));
            } else {
                height = rnd(72.0*(log10((double)line[i*npix+j]+1.0)-(double)gmin)/(double)(gmax-gmin));
            }
            MoveTo(i*(npix+70)+j+40,82);
            LineTo(i*(npix+70)+j+40,82-height);
        }
        ForeColor(blackColor);
    }
    ClosePicture();
    SetPort(port);
    MESSAGE('P',(char *)p);
#else
    /* plot ordinate axis */
    MESSAGE('I',"");
    for (i=0; i<AXIS/2; msg[i++]=' ');
    for (msg[i]='\0',j=0; j<min(3,nbnd); j++) {
        if (option == UNIT  ||  option == NORM) {
            sprintf(msg+strlen(msg),"%*.4e",AXIS,(double)gmin);
            sprintf(msg+strlen(msg),"%*.4e",AXIS,(double)gmax);
        } else {
            sprintf(msg+strlen(msg),"%*.4e",AXIS,pow(10.0,(double)gmin-1.0));
            sprintf(msg+strlen(msg),"%*.4e",AXIS,pow(10.0,(double)gmax-1.0));
        }
    }
    MESSAGE('I',msg);
    sprintf(msg," ");
    for (i=0; i<min(3,nbnd); i++) {
        sprintf(msg+strlen(msg),"%s%s",axis[0],axis[1]);
    }
    sprintf(msg," %-.*s",AXIS-2,minxlbl);
    msg[strlen(msg)] = ' ';
    MESSAGE('I',msg);

    /* plot the line data */
    for (i=0; i<npix; i++) {
        sprintf(msg," ");
        for (j=0; j<min(3,nbnd); j++) {
            sprintf(msg+strlen(msg),"           |           |");
            if (option == UNIT) {
                height = max(0,min(rnd((double)(AXIS)*(double)(line[j*npix+i]-gmin)/(double)(gmax-gmin)),AXIS));
            } else if (option == NORM) {
                height = rnd((double)(AXIS)*(double)(line[j*npix+i]-gmin)/(double)(gmax-gmin));
            } else {
                height = rnd((double)(AXIS)*(log10((double)line[j*npix+i]+1.0)-(double)gmin)/(double)(gmax-gmin));
            }
            msg[(j*2+1)*AXIS+height] = '*';
        }
        MESSAGE('I',msg);
    }

    /* plot ordinate axis */
    sprintf(msg," ");
    for (i=0; i<min(3,nbnd); i++) {
        sprintf(msg+strlen(msg),"%s%s",axis[0],axis[1]);
    }
    sprintf(msg," %-.*s",AXIS-2,maxxlbl);
    msg[strlen(msg)] = ' ';
    MESSAGE('I',msg);
    for (i=0; i<AXIS/2; msg[i++]=' ');
    for (msg[i]='\0',j=0; j<min(3,nbnd); j++) {
        if (option == UNIT  ||  option == NORM) {
            sprintf(msg+strlen(msg),"%*.4e",AXIS,(double)gmin);
            sprintf(msg+strlen(msg),"%*.4e",AXIS,(double)gmax);
        } else {
            sprintf(msg+strlen(msg),"%*.4e",AXIS,pow(10.0,(double)gmin-1.0));
            sprintf(msg+strlen(msg),"%*.4e",AXIS,pow(10.0,(double)gmax-1.0));
        }
    }
    MESSAGE('I',msg);
    MESSAGE('I',"");
#endif

    the_end:
    if (TIMES) TIMING(T_EXIT);
    return(0);
}
