#ifdef HAVE_CONFIG_H
#include <config.h>
#endif /* HAVE_CONFIG_H */
#include        "sadie.h"

/*-Copyright Information------------------------------------------------------*/
/* Copyright (c) 1988 by the University of Arizona Digital Image Analysis Lab */
/*----------------------------------------------------------------------------*/
/*-General Information--------------------------------------------------------*/
/*                                                                            */
/*   This function computes the root-mean-squared difference between          */
/*   two images.                                                              */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/*-Interface Information------------------------------------------------------*/
void RMSDIFF (
IMAGE  *in1,    /*  I   Pointer to the first input image.                     */
IMAGE  *in2,    /*  I   Pointer to the second input image.                    */
double *diff    /*  O   Address of a variable to hold the rms-difference.     */
/*----------------------------------------------------------------------------*/
) { register uint32_t i, j, k;
    char msg[SLEN];

    if (TIMES) TIMING(T_RMSDIFF);
    if (NAMES) {
        MESSAGE('I',"");
        MESSAGE('I',"RMSDIFF");
    }

    /* check input */
    if (!CHECKIMG(in1)) {
        MESSAGE('E'," Can't identify first image.");
        goto the_end;
    } else if (!CHECKIMG(in2)) {
        MESSAGE('E'," Can't identify second image.");
        goto the_end;
    } else if (in1->nbnd != in2->nbnd) {
        MESSAGE('E'," Number of bands must be identical in both images.");
        goto the_end;
    } else if (in1->nlin != in2->nlin) {
        MESSAGE('E'," Number of lines must be identical in both images.");
        goto the_end;
    } else if (in1->npix != in2->npix) {
        MESSAGE('E'," Number of pixels/line must be identical in both images.");
        goto the_end;
    }

    if (NAMES) {
        MESSAGE('I',"");
        sprintf(msg," First input image:    %s",in1->text);
        MESSAGE('I',msg);
        sprintf(msg," Second input image:   %s",in2->text);
        MESSAGE('I',msg);
        MESSAGE('I'," ...............");
    }

    /* calculate the root-mean-squared difference */
    for (*diff=0.0,i=0; i<in1->nbnd; i++) {
        for (j=0; j<in1->nlin; j++) {
            for (k=0; k<in1->npix; k++) {
                *diff += (double)((in1->data[i][j][k]-in2->data[i][j][k])*(in1->data[i][j][k]-in2->data[i][j][k]));
            }
        }
    }
    *diff = sqrt(*diff/((double)in1->nbnd*(double)in1->nlin*(double)in1->npix));

    /* output the resulting difference */
    MESSAGE('I',"");
    sprintf(msg," Root-Mean-Squared Difference = %-11.4e",*diff);
    MESSAGE('I',msg);

    the_end:
    if (TIMES) TIMING(T_EXIT);
}
