#ifdef HAVE_CONFIG_H
#include <config.h>
#endif /* HAVE_CONFIG_H */
#include        "sadie.h"

#define  BAND   0
#define  LINE   1
#define  PIXEL  2

/*-Copyright Information------------------------------------------------------*/
/* Copyright (c) 1999 by the University of Arizona Digital Image Analysis Lab */
/*----------------------------------------------------------------------------*/
/*-General Information--------------------------------------------------------*/
/*                                                                            */
/*   This function rotates image bands, lines, and pixels.                    */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/*-Interface Information------------------------------------------------------*/
void ROTATECUBE (
IMAGE  *in,     /*  I   Pointer to the input image.                           */
short  xdir,    /*  I   Switches, indicating direction of index of axis:      */
short  ydir,    /*  I   0 - Increasing index.                                 */
short  zdir,    /*  I   1 - Decreasing index.                                 */
short  xaxis,   /*  I   Switches, indicating order of axis:                   */
short  yaxis,   /*  I   0 - Band.                                             */
short  zaxis,   /*  I   1 - Line.                                             */
                /*      2 - Pixel.                                            */
IMAGE  **out    /*  O   Address of a pointer to the output image.             */
/*----------------------------------------------------------------------------*/
) { register uint32_t i, j, k, outi, outj, outk;
    char msg[SLEN];

    if (TIMES) TIMING(T_ROTATECUBE);
    if (NAMES) {
        MESSAGE('I',"");
        MESSAGE('I',"ROTATECUBE");
    }

    /* check input */
    if (!CHECKIMG(in)) {
        MESSAGE('E'," Can't identify input image.");
        goto the_end;
    } else if (xaxis == yaxis  ||  yaxis == zaxis  ||  zaxis == xaxis) {
        MESSAGE('E'," Can't map between same axis.");
        goto the_end;
    }

    if (NAMES) {
        MESSAGE('I',"");
        sprintf(msg," Input image:        %s",in->text);
        MESSAGE('I',msg);
        MESSAGE('I',"");
        MESSAGE('I'," Output format:");
        switch (zaxis) {
        case BAND:
            switch (yaxis) {
            case LINE:
                sprintf(msg,"  X Axis: %c Pixel",(xdir==0?'+':'-'));
                MESSAGE('I',msg);
                sprintf(msg,"  Y Axis: %c Line",(ydir==0?'+':'-'));
                MESSAGE('I',msg);
                sprintf(msg,"  Z Axis: %c Band",(zdir==0?'+':'-'));
                MESSAGE('I',msg);
                break;
            case PIXEL:
                sprintf(msg,"  X Axis: %c Line",(xdir==0?'+':'-'));
                MESSAGE('I',msg);
                sprintf(msg,"  Y Axis: %c Pixel",(ydir==0?'+':'-'));
                MESSAGE('I',msg);
                sprintf(msg,"  Z Axis: %c Band",(zdir==0?'+':'-'));
                MESSAGE('I',msg);
                break;
            }
            break;
        case LINE:
            switch (yaxis) {
            case BAND:
                sprintf(msg,"  X Axis: %c Pixel",(xdir==0?'+':'-'));
                MESSAGE('I',msg);
                sprintf(msg,"  Y Axis: %c Band",(ydir==0?'+':'-'));
                MESSAGE('I',msg);
                sprintf(msg,"  Z Axis: %c Line",(zdir==0?'+':'-'));
                MESSAGE('I',msg);
                break;
            case PIXEL:
                sprintf(msg,"  X Axis: %c Band",(xdir==0?'+':'-'));
                MESSAGE('I',msg);
                sprintf(msg,"  Y Axis: %c Pixel",(ydir==0?'+':'-'));
                MESSAGE('I',msg);
                sprintf(msg,"  Z Axis: %c Line",(zdir==0?'+':'-'));
                MESSAGE('I',msg);
                break;
            }
            break;
        case PIXEL:
            switch (yaxis) {
            case BAND:
                sprintf(msg,"  X Axis: %c Line",(xdir==0?'+':'-'));
                MESSAGE('I',msg);
                sprintf(msg,"  Y Axis: %c Band",(ydir==0?'+':'-'));
                MESSAGE('I',msg);
                sprintf(msg,"  Z Axis: %c Pixel",(zdir==0?'+':'-'));
                MESSAGE('I',msg);
                break;
            case LINE:
                sprintf(msg,"  X Axis: %c Band",(xdir==0?'+':'-'));
                MESSAGE('I',msg);
                sprintf(msg,"  Y Axis: %c Line",(ydir==0?'+':'-'));
                MESSAGE('I',msg);
                sprintf(msg,"  Z Axis: %c Pixel",(zdir==0?'+':'-'));
                MESSAGE('I',msg);
                break;
            }
            break;
        }
    }
    MESSAGE('I'," ...............");

    /* switch to decide required cube orientation */
    switch (zaxis) {
    case BAND:
        switch (yaxis) {
        case LINE:
            /* create image of appropriate size */
            if (!CHECKIMG(*out)) GETMEM(in->nbnd,in->nlin,in->npix,out);
            if (!*out) goto the_end;
            /* rotate cube */
            for(i=0; i<in->nbnd; i++) {
                for(j=0; j<in->nlin; j++) {
                    for(k=0; k<in->npix; k++) {
                        outi = abs(zdir*(in->nbnd-1)-i);
                        outj = abs(ydir*(in->nlin-1)-j);
                        outk = abs(xdir*(in->npix-1)-k);
                        (*out)->data[outi][outj][outk] = in->data[i][j][k];
                    }
                }
            }
            break;
        case PIXEL:
            /* create image of appropriate size */
            if (!CHECKIMG(*out)) GETMEM(in->nbnd,in->npix,in->nlin,out);
            if (!*out) goto the_end;
            /* rotate cube */
            for(i=0; i<in->nbnd; i++) {
                for(j=0; j<in->nlin; j++) {
                    for(k=0; k<in->npix; k++) {
                        outi = abs(zdir*(in->nbnd-1)-i);
                        outj = abs(ydir*(in->npix-1)-k);
                        outk = abs(xdir*(in->nlin-1)-j);
                        (*out)->data[outi][outj][outk] = in->data[i][j][k];
                    }
                }
            }
            break;
        }
        break;
    case LINE:
        switch (yaxis) {
        case BAND:
            /* create image of appropriate size */
            if (!CHECKIMG(*out)) GETMEM(in->nlin,in->nbnd,in->npix,out);
            if (!*out) goto the_end;
            /* rotate cube */
            for(i=0; i<in->nbnd; i++) {
                for(j=0; j<in->nlin; j++) {
                    for(k=0; k<in->npix; k++) {
                        outi = abs(zdir*(in->nlin-1)-j);
                        outj = abs(ydir*(in->nbnd-1)-i);
                        outk = abs(xdir*(in->npix-1)-k);
                        (*out)->data[outi][outj][outk] = in->data[i][j][k];
                    }
                }
            }
            break;
        case PIXEL:
            /* create image of appropriate size */
            if (!CHECKIMG(*out)) GETMEM(in->nlin,in->npix,in->nbnd,out);
            if (!*out) goto the_end;
            /* rotate cube */
            for(i=0; i<in->nbnd; i++) {
                for(j=0; j<in->nlin; j++) {
                    for(k=0; k<in->npix; k++) {
                        outi = abs(zdir*(in->nlin-1)-j);
                        outj = abs(ydir*(in->npix-1)-k);
                        outk = abs(xdir*(in->nbnd-1)-i);
                        (*out)->data[outi][outj][outk] = in->data[i][j][k];
                    }
                }
            }
            break;
        }
        break;
    case PIXEL:
        switch (yaxis) {
        case BAND:
            /* create image of appropriate size */
            if (!CHECKIMG(*out)) GETMEM(in->npix,in->nbnd,in->nlin,out);
            if (!*out) goto the_end;
            /* rotate cube */
            for(i=0; i<in->nbnd; i++) {
                for(j=0; j<in->nlin; j++) {
                    for(k=0; k<in->npix; k++) {
                        outi = abs(zdir*(in->npix-1)-k);
                        outj = abs(ydir*(in->nbnd-1)-i);
                        outk = abs(xdir*(in->nlin-1)-j);
                        (*out)->data[outi][outj][outk] = in->data[i][j][k];
                    }
                }
            }
            break;
        case LINE:
            /* create image of appropriate size */
            if (!CHECKIMG(*out)) GETMEM(in->npix,in->nlin,in->nbnd,out);
            if (!*out) goto the_end;
            /* rotate cube */
            for(i=0; i<in->nbnd; i++) {
                for(j=0; j<in->nlin; j++) {
                    for(k=0; k<in->npix; k++) {
                        outi = abs(zdir*(in->npix-1)-k);
                        outj = abs(ydir*(in->nlin-1)-j);
                        outk = abs(xdir*(in->nbnd-1)-i);
                        (*out)->data[outi][outj][outk] = in->data[i][j][k];
                    }
                }
            }
            break;
        }
        break;
    }

    the_end:
    if (TIMES) TIMING(T_EXIT);
}









