#ifdef HAVE_CONFIG_H
#include <config.h>
#endif /* HAVE_CONFIG_H */
#include        "sadie.h"

/*-Copyright Information------------------------------------------------------*/
/* Copyright (c) 1988 by the University of Arizona Digital Image Analysis Lab */
/*----------------------------------------------------------------------------*/
/*-General Information--------------------------------------------------------*/
/*                                                                            */
/*   This function generates a sinewave pattern in the x-direction.           */
/*   The minimum graylevel is SMIN, and the maximum graylevel is SMAX.        */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/*-Interface Information------------------------------------------------------*/
void SINEWAVE (
uint32_t nlin,  /*  I   Number of lines.                                      */
uint32_t npix,  /*  I   Number of pixels/line.                                */
double period,  /*  I   Period of the sinewave in pixels.                     */
double phase,   /*  I   Phase of the sinewave in pixels.                      */
IMAGE  **out    /*  O   Address of a pointer to the output image.             */
/*----------------------------------------------------------------------------*/
) { register uint32_t j, k;
    register PIXEL p;
    char   msg[SLEN];
    double factor;

    if (TIMES) TIMING(T_SINEWAVE);
    if (NAMES) {
        MESSAGE('I',"");
        MESSAGE('I',"SINEWAVE");
        MESSAGE('I',"");
        sprintf(msg," Image size: lines:    %lu",nlin);
        MESSAGE('I',msg);
        sprintf(msg,"             pixels:   %lu",npix);
        MESSAGE('I',msg);
        sprintf(msg," Period:             %12.4e",period);
        MESSAGE('I',msg);
        sprintf(msg," Phase:              %12.4e",phase);
        MESSAGE('I',msg);
        MESSAGE('I'," ...............");
    }

    /* check input */
    if (nlin <= 0) {
        MESSAGE('E'," Number of lines must be greater than zero.");
        goto the_end;
    } else if (npix <= 0) {
        MESSAGE('E'," Number of pixels/line must be greater than zero.");
        goto the_end;
    } else if (period <= 0.0) {
        MESSAGE('E'," Period must be greater than zero.");
        goto the_end;
    }

    /* create image of appropriate size */
    if (!CHECKIMG(*out)) GETMEM(1,nlin,npix,out);
    if (!*out) goto the_end;

    /* create sinewave pattern */
    factor = 2.0*PI/period;
    phase *= factor;
    for (k=0; k<npix; k++) {
        p = (PIXEL)((double)(SMAX-SMIN)*(0.5*sin(factor*(double)k+phase)+0.5)+(double)SMIN);
        for (j=0; j<nlin; j++) {
            (*out)->data[0][j][k] = p;
        }
    }
    
    the_end:
    if (TIMES)  TIMING(T_EXIT);
}
