#ifdef HAVE_CONFIG_H
#include <config.h>
#endif /* HAVE_CONFIG_H */
#include        "sadie.h"

/*-Copyright Information------------------------------------------------------*/
/* Copyright (c) 1988 by the University of Arizona Digital Image Analysis Lab */
/*----------------------------------------------------------------------------*/
/*-General Information--------------------------------------------------------*/
/*                                                                            */
/*   This function transforms the input image according to the given option.  */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/*-Interface Information------------------------------------------------------*/
void TRNSFORM (
IMAGE *in,      /*  I   Pointer to the input image                            */
short option,   /*  I   Switch, indicating the type of transformation:        */
                /*      FLIP0   -   Mirror axis vertical.                     */
                /*      FLIPM45 -   Mirror axis -45 degree diagonal.          */
                /*      FLIPP45 -   Mirror axis +45 degree diagonal.          */
                /*      FLIP90  -   Mirror axis horizontal.                   */
                /*      ROTPM0  -   Rotate +/-0 degrees.                      */
                /*      ROTP90  -   Rotate +90 degrees.                       */
                /*      ROTM90  -   Rotate -90 degrees.                       */
                /*      ROT180  -   Rotate 180 degrees.                       */
IMAGE **out     /*  O   Address of a pointer to the output image              */
/*----------------------------------------------------------------------------*/
) { register uint32_t i, j1, k1, j2, k2;
    char msg[SLEN];

    if (TIMES) TIMING(T_TRNSFORM);
    if (NAMES) {
        MESSAGE('I',"");
        MESSAGE('I',"TRNSFORM");
    }

    /* check input */
    if (!CHECKIMG(in)) {
        MESSAGE('E'," Can't identify image");
        goto the_end;
    } else if (option != FLIP0  &&  option != FLIPP45  &&  option != FLIPM45  &&option != FLIP90  &&  option != ROTPM0  &&  option != ROTP90  &&  option != ROTM90  &&  option != ROT180) {
        MESSAGE('E'," Option must be one of FLIP0, FLIPP45, FLIPM45, FLIP90, ROTPM0, ROTP90, ROTM90, and ROT180.");
        goto the_end;
    }

    if (NAMES) {
        MESSAGE('I',"");
        sprintf(msg," Input image:   %s",in->text);
        MESSAGE('I',msg);
        if (option == FLIP0) {
            MESSAGE('I'," Mirror axis vertical");
        } else if (option == FLIPM45) {
            MESSAGE('I'," Mirror axis -45 degree diagonal");
        } else if (option == FLIPP45) {
            MESSAGE('I'," Mirror axis +45 degree diagonal");
        } else if (option == FLIP90) {
            MESSAGE('I'," Mirror axis horizontal");
        } else if (option == ROTPM0) {
            MESSAGE('I'," Rotate +/-0 degrees");
        } else if (option == ROTP90) {
            MESSAGE('I'," Rotate +90 degrees");
        } else if (option == ROTM90) {
            MESSAGE('I'," Rotate -90 degrees");
        } else if (option == ROT180) {
            MESSAGE('I'," Rotate 180 degrees");
        }
        MESSAGE('I'," ...............");
    }

    /* transform image */
    switch (option) {
    case FLIP0:
        if (!CHECKIMG(*out)) GETMEM(in->nbnd,in->nlin,in->npix,out);  
        if (!*out) goto the_end;
        for (i=0; i<in->nbnd; i++) {
            for (j1=0; j1<in->nlin; j1++) {
                for (k1=in->npix-1, k2=0; k2<in->npix; k1--,k2++) {
                    (*out)->data[i][j1][k2] = in->data[i][j1][k1];
                }
            }
        }
        break;
    case FLIPM45:
        if (!CHECKIMG(*out)) GETMEM(in->nbnd,in->npix,in->nlin,out);  
        if (!*out) goto the_end;
        for (i=0; i<in->nbnd; i++) {
            for (j1=0; j1<in->nlin; j1++) {
                for (k1=0; k1<in->npix; k1++) {
                    (*out)->data[i][k1][j1] = in->data[i][j1][k1];
                }
            }
        }
        break;
    case FLIPP45:
        if (!CHECKIMG(*out)) GETMEM(in->nbnd,in->npix,in->nlin,out);  
        if (!*out) goto the_end;
        for (i=0; i<in->nbnd; i++) {
            for (j1=0,j2=in->nlin-1; j1<in->nlin; j1++,j2--) {
                for (k1=0,k2=in->npix-1; k1<in->npix; k1++,k2--) {
                    (*out)->data[i][k1][j2] = in->data[i][j1][k2];
                }
            }
        }
        break;
   case FLIP90:
        if (!CHECKIMG(*out)) GETMEM(in->nbnd,in->nlin,in->npix,out);  
        if (!*out) goto the_end;
        for (i=0; i<in->nbnd; i++) {
            for (j1=in->nlin-1,j2=0; j2<in->nlin; j1--,j2++) {
                for (k1=0; k1<in->npix; k1++) {
                    (*out)->data[i][j2][k1] = in->data[i][j1][k1];
                }
            }
        }
        break;
    case ROTPM0:
        if (!CHECKIMG(*out)) GETMEM(in->nbnd,in->nlin,in->npix,out);
        if (!*out) goto the_end;
        for (i=0; i<in->nbnd; i++) {
            for (j1=j2=0; j1<in->nlin; j1++,j2++) {
                for (k1=k2=0; k1<in->npix; k1++,k2++) {
                    (*out)->data[i][j2][k2] = in->data[i][j1][k1];
                }
            }
        }
        break;
    case ROTP90:
        if (!CHECKIMG(*out)) GETMEM(in->nbnd,in->npix,in->nlin,out);
        if (!*out) goto the_end;
        for (i=0; i<in->nbnd; i++) {
            for (k1=j2=0; k1<in->npix; k1++,j2++) {
                for (j1=in->nlin-1,k2=0; k2<in->nlin; j1--,k2++) {
                    (*out)->data[i][j2][k2] = in->data[i][j1][k1];
                }
            }
        }
        break;
    case ROTM90:
        if (!CHECKIMG(*out)) GETMEM(in->nbnd,in->npix,in->nlin,out);
        if (!*out) goto the_end;
        for (i=0; i<in->nbnd; i++) {
            for (k1=in->npix-1,j2=0; j2<in->npix; k1--,j2++) {
                for (j1=k2=0; j1<in->nlin; j1++,k2++) {
                     (*out)->data[i][j2][k2] = in->data[i][j1][k1];
                }
            }
        }
        break;
    case ROT180:
        if (!CHECKIMG(*out)) GETMEM(in->nbnd,in->nlin,in->npix,out);
        if (!*out) goto the_end;
        for (i=0; i<in->nbnd; i++) {
            for (j1=in->nlin-1,j2=0; j2<in->nlin; j1--,j2++) {
                for (k1=in->npix-1,k2=0; k2<in->npix; k1--,k2++) {
                    (*out)->data[i][j2][k2] = in->data[i][j1][k1];
                }
            }
        }
        break;
    }

    (*out)->gmin = in->gmin;
    (*out)->gmax = in->gmax;

    the_end:
    if (TIMES) TIMING(T_EXIT);
}
